import dataclasses

# Public interface.
__all__ = ["GoposAddress"]

from typing import Optional


@dataclasses.dataclass
class GoposAddress:
    """
    Response of a request to the `GET /address/parse` route.

    Definition of the fields is available at:
    https://github.com/openvenues/libpostal#parser-labels.
    """

    category: Optional[str] = None
    city: Optional[str] = None
    city_district: Optional[str] = None
    country: Optional[str] = None
    country_region: Optional[str] = None
    entrance: Optional[str] = None
    house: Optional[str] = None
    house_number: Optional[str] = None
    island: Optional[str] = None
    level: Optional[str] = None
    near: Optional[str] = None
    po_box: Optional[str] = None
    postcode: Optional[str] = None
    road: Optional[str] = None
    staircase: Optional[str] = None
    state: Optional[str] = None
    state_district: Optional[str] = None
    suburb: Optional[str] = None
    unit: Optional[str] = None
    world_region: Optional[str] = None



import dataclasses

# Public interface.
__all__ = ["TimezoneResponse"]


@dataclasses.dataclass
class TimezoneResponse:
    """
    Response of a request to the `GET /timezone` route.
    """

    latitude: float
    longitude: float
    timezone: str



import dataclasses

# Public interface.
__all__ = ["HealthResponse"]


@dataclasses.dataclass
class HealthResponse:
    """
    Response of a request to the `GET /health` route.
    """

    status: bool



from typing import Any, Dict
from urllib.parse import quote_plus

import dacite
import requests

from gopos.models import GoposAddress, HealthResponse, TimezoneResponse

# Public interface.
__all__ = ["GoposClient"]


class GoposClient:
    def __init__(self, base: str):
        """
        GoposClient constructor.

        :param base: url at which the Gopos server is running
        """
        self._base = base

    def _call(self, endpoint: str) -> Dict[str, Any]:
        """
        Calls the service.

        :param endpoint: the endpoint to call
        :return: the response parsed as JSON
        """
        # Send the request.
        url = f"{self._base}{endpoint}"
        request = requests.get(url)

        # Error handling.
        request.raise_for_status()

        # Parse the response.
        return request.json()

    def get_health(self) -> bool:
        """
        Gets the health check response.

        :return: true if the service is healthy
        """
        # Send the request.
        data = self._call("/health")
        parsed = dacite.from_dict(HealthResponse, data)

        # Extract the status.
        return parsed.status

    def get_place_details(self, place_id: str) -> Dict[str, Any]:
        """
        Gets the details of the given Google Place ID.

        :param place_id: the ID to lookup
        :return: the details
        """
        # Send the request.
        return self._call(f"/google/place/{place_id}")

    def get_timezone(self, latitude: float, longitude: float) -> str:
        """
        Gets the timezone for the given coordinate pair.

        :param latitude: the latitude coordinate
        :param longitude: the longitude coordinate
        :return: the timezone
        """
        # Send the request.
        data = self._call(f"/timezone?latitude={latitude}&longitude={longitude}")
        parsed = dacite.from_dict(TimezoneResponse, data)

        # Extract the timezone.
        return parsed.timezone

    def parse_address(self, query: str, country: str = None) -> GoposAddress:
        """
        Parses the address for the given query.

        :param query: the address to parse
        :param country: (optional) if passed, this will help the address parser
               to obtain more accurate results
        :return: the address
        """
        # Build the endpoint.
        endpoint = f"/address/parse?query={quote_plus(query)}"
        if country:
            endpoint = f"{endpoint}&country={country}"

        # Send the request.
        data = self._call(endpoint)
        return dacite.from_dict(GoposAddress, data)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['gopos', 'gopos.models']

package_data = \
{'': ['*']}

install_requires = \
['dacite>=1.2.0,<2.0.0', 'requests>=2.21.0,<3.0.0']

setup_kwargs = {
    'name': 'gopos',
    'version': '1.0.0',
    'description': 'Python bindings for gopos (https://github.com/thepieterdc/gopos).',
    'long_description': '# gopos-python\nPython bindings for gopos (https://github.com/thepieterdc/gopos).\n',
    'author': 'Pieter De Clercq',
    'author_email': 'pieterdeclercq@outlook.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/thepieterdc/gopos-python',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
